@@ -1,32 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2008-2012 NVIDIA Corporation.
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2008-2012 NVIDIA Corporation.
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QSSG_RENDER_CAMERA_H
 #define QSSG_RENDER_CAMERA_H
@@ -59,6 +33,14 @@ struct QSSGCameraGlobalCalculationResult
 
 struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderCamera : public QSSGRenderNode
 {
+    enum class DirtyFlag : quint8
+    {
+        CameraDirty = 0x1
+    };
+    using FlagT = std::underlying_type_t<DirtyFlag>;
+
+    static constexpr DirtyFlag DirtyMask { std::numeric_limits<FlagT>::max() };
+
     // Setting these variables should set dirty on the camera.
     float clipNear;
     float clipFar;
@@ -81,6 +63,9 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderCamera : public QSSGRenderNode
     // to use during mouse picking.
     QVector2D frustumScale;
     bool enableFrustumClipping;
+    FlagT cameraDirtyFlags = 0;
+
+    float levelOfDetailPixelThreshold = 1.0;
 
     QRectF previousInViewport;
 
@@ -92,7 +77,7 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderCamera : public QSSGRenderNode
     // Need to test this when the camera's local transform is null.
     // Assumes parent's local transform is the identity, meaning our local transform is
     // our global transform.
-    void lookAt(const QVector3D &inCameraPos, const QVector3D &inUpDir, const QVector3D &inTargetPos);
+    void lookAt(const QVector3D &inCameraPos, const QVector3D &inUpDir, const QVector3D &inTargetPos, const QVector3D &pivot);
 
     QSSGCameraGlobalCalculationResult calculateGlobalVariables(const QRectF &inViewport);
     bool calculateProjection(const QRectF &inViewport);
@@ -107,6 +92,8 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderCamera : public QSSGRenderNode
 
     void calculateViewProjectionMatrix(QMatrix4x4 &outMatrix) const;
 
+    void calculateViewProjectionWithoutTranslation(float near, float far, QMatrix4x4 &outMatrix) const;
+
     // Unproject a point (x,y) in viewport relative coordinates meaning
     // left, bottom is 0,0 and values are increasing right,up respectively.
     QSSGRenderRay unproject(const QVector2D &inLayerRelativeMouseCoords, const QRectF &inViewport) const;
@@ -118,6 +105,16 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderCamera : public QSSGRenderNode
 
     float verticalFov(float aspectRatio) const;
     float verticalFov(const QRectF &inViewport) const;
+
+    [[nodiscard]] inline bool isDirty(DirtyFlag dirtyFlag = DirtyMask) const
+    {
+        return ((cameraDirtyFlags & FlagT(dirtyFlag)) != 0)
+               || ((dirtyFlag == DirtyMask) && QSSGRenderNode::isDirty());
+    }
+    void markDirty(DirtyFlag dirtyFlag);
+    void clearDirty(DirtyFlag dirtyFlag);
+
+    float getLevelOfDetailMultiplier() const;
 };
 
 QT_END_NAMESPACE
